#ifndef SOPHIE_CORE_DOT_H_INCLUDED
#define SOPHIE_CORE_DOT_H_INCLUDED

typedef struct tag_SaviErrInfo {
    U32 errCode;
    char *errText;
    int resultCode;
} SaviErrInfo;

#define ERROR_ARRAY_END_MARKER ((U32)0xffffffff)

SaviErrInfo errInfo[]={

{ SOPHOS_SAVI_ERROR_INITIALISING,         "error: SAVI initialization failed", -1 },
{ SOPHOS_SAVI_ERROR_TERMINATING,          "error: Error occured while unloading SAVI", -1 },
{ SOPHOS_SAVI_ERROR_SWEEPFAILURE,         "error: Virus scan failed", -1 },
{ SOPHOS_SAVI_ERROR_VIRUSPRESENT,         "A virus was detected", 1 },
{ SOPHOS_SAVI_ERROR_NOT_INITIALISED,      "error: Engine not initialized", -1 },
{ SOPHOS_SAVI_ERROR_IC_INCOMPATIBLE_VERSION, "error: InterCheck client not compatible with this version of SAV", -1 },
{ SOPHOS_SAVI_ERROR_IC_ACCESS_DENIED,     "error: Insufficient right - unable to disable InterCheck client", -1 },
{ SOPHOS_SAVI_ERROR_IC_SCAN_PREVENTED,    "error: InterCheck client could not be disabled.  Request to scan the file has been denied.", -1 },
{ SOPHOS_SAVI_ERROR_DISINFECTION_FAILED,  "error: Disinfection failed", -1 },
{ SOPHOS_SAVI_ERROR_DISINFECTION_UNAVAILABLE, "error: Disinfection attempted on an uninfected file", -1 },
{ SOPHOS_SAVI_ERROR_UPGRADE_FAILED,       "error: Attempt to upgrade virus engine failed", -1 },
{ SOPHOS_SAVI_ERROR_SAV_NOT_INSTALLED,    "error: Sophos Anti-Virus is not installed", -1 },
{ SOPHOS_SAVI_ERROR_INVALID_CONFIG_NAME,  "error: Attempt to get/set SAVI configuration with incorrect name", -1 },
{ SOPHOS_SAVI_ERROR_INVALID_CONFIG_TYPE,  "error: Attempt to get/set SAVI configuration with incorrect type", -1 },
{ SOPHOS_SAVI_ERROR_INIT_CONFIGURATION,   "error: Could not configure SAVI", -1 },
{ SOPHOS_SAVI_ERROR_NOT_SUPPORTED,        "error: Not supported in this SAVI implementation", 0 },
{ SOPHOS_SAVI_ERROR_COULD_NOT_OPEN,       "error: File could not be accessed", -1 },
{ SOPHOS_SAVI_ERROR_FILE_COMPRESSED,      "error: File was compressed, but no virus was found on the outer level",0 },
{ SOPHOS_SAVI_ERROR_FILE_ENCRYPTED,       "error: File was encrypted", 0 },
{ SOPHOS_SAVI_ERROR_INFORMATION_NOT_AVAILABLE, "error: Additional virus location is unavailable (whatever that means ;)", -1 },
{ SOPHOS_SAVI_ERROR_ALREADY_INIT,         "error: Attempt to initialize SAVI when it is already initialized", -1 },
{ SOPHOS_SAVI_ERROR_STUB,                 "error: Attempt to use stub library", -1 },
{ SOPHOS_SAVI_ERROR_BUFFER_TOO_SMALL,     "error: Buffer supplied was too small", -1 },
{ SOPHOS_SAVI_CBCK_CONTINUE_THIS,         "error: Returned from a callback function to continue with the current file.", -1 },
{ SOPHOS_SAVI_CBCK_CONTINUE_NEXT,         "error: Returned from a callback function to skip to the next file.", -1 },
{ SOPHOS_SAVI_CBCK_STOP,                  "error: Returned from a callback function to stop the current operation.", -1 },
{ SOPHOS_SAVI_ERROR_CORRUPT,              "error: File corrupted, Sophie could not proceed.", 0 },
{ SOPHOS_SAVI_ERROR_REENTRANCY,           "error: An attempt to re-enter SAVI from a callback notification was detected", -1 },
{ SOPHOS_SAVI_ERROR_CALLBACK,             "error: An error was encountered in the SAVI client's callback function.", -1 },
{ SOPHOS_SAVI_ERROR_PARTIAL_INFORMATION,  "error: A call requesting several pieces of information did not return them all.", -1 },
{ SOPHOS_SAVI_ERROR_OLD_VIRUS_DATA,       "error: The main body of virus data is out of date.", -1 },
{ SOPHOS_SAVI_ERROR_INVALID_TMP,          "error: No valid temporary directory found.", -1 },
{ SOPHOS_SAVI_ERROR_MISSING_MAIN_VIRUS_DATA, "error: The main body of virus data is missing.", -1 },
{ SOPHOS_SAVI_INFO_IC_ACTIVE,             "error: The InterCheck client is active, and could not be disabled.", -1 },
{ SOPHOS_SAVI_ERROR_VIRUS_DATA_INVALID_VER, "error: The virus data main body has an invalid version.", -1 },
{ SOPHOS_SAVI_ERROR_MUST_REINIT,          "error: SAVI must be reinitialized (the virus engine has a version higher than the running version of SAVI supports.", -1 },
{ SOPHOS_SAVI_ERROR_CANNOT_SET_OPTION,    "error: Cannot set option value - the virus engine will not permit its value to be changed, as this option is immutable.", -1 },
{ SOPHOS_SAVI_ERROR_PART_VOL,             "error: The file passed for scanning represented part of a multi volume archive. The file cannot be scanned.", 0 },
{ SOPHOS_SAVI_INFO_OPT_GRP_INVAL_RTN,     "error: Attempt to read value of group config item", -1 },
{ SOPHOS_SAVI_ERROR_VDLD_ACTIVITY,        "error: Atempt to scan and update data simultaneously", -1 },
{ SOPHOS_SAVI_ERROR_MISSING_VDL_PART,     "error: One or more parts of the VDL data is missing", -1 },
{ SOPHOS_SAVI_WARNING_MISSING_VDL_PART,   "warning: One or more parts of the VDL data is missing", -1 },
{ SOPHOS_SAVI_ERROR_VDL_CHECKSUM,         "error: One of the virus data files has an invalid checksum", -1 },
{ SOPHOS_SAVI_WARNING_VDL_CHECKSUM,       "warning: One of the virus data files has an invalid checksum", -1 },
{ SOPHOS_SAVI_ERROR_SCAN_ABORTED,         "error: Scan aborted - suspected zip bomb", 0 },
{ ERROR_ARRAY_END_MARKER,                 "error: Default error case.", -1 }
};
#endif
